@@ -6,6 +6,7 @@ import pydantic as pd
 import tidy3d as td
 from tidy3d.exceptions import Tidy3dError
 from tidy3d.log import DEFAULT_LEVEL, _get_level_int, set_logging_level
+from ..utils import log_capture, assert_log_level
 
 
 def test_log():
@@ -39,3 +40,16 @@ def test_exception_message():
     MESSAGE = "message"
     e = Tidy3dError(MESSAGE)
     assert str(e) == MESSAGE
+
+
+def test_logging_upper():
+    """Make sure we get an error if lowercase."""
+    td.config.logging_level = "WARNING"
+    with pytest.raises(ValueError):
+        td.config.logging_level = "warning"
+
+
+def test_logging_unrecognized():
+    """If unrecognized option, raise validation errorr."""
+    with pytest.raises(pd.ValidationError):
+        td.config.logging_level = "blah"
