@@ -5,7 +5,7 @@ import gdstk
 
 import tidy3d as td
 
-from tidy3d.plugins import ComplexPolySlab
+from tidy3d.plugins.polyslab import ComplexPolySlab
 from ..utils import clear_tmp, assert_log_level, log_capture
 
 
@@ -58,7 +58,7 @@ def test_many_sub_polyslabs(log_capture):
         geometry=s.geometry_group,
         medium=td.Medium(permittivity=2),
     )
-    assert_log_level(log_capture, "warning")
+    assert_log_level(log_capture, "WARNING")
 
 
 def test_divide_simulation():
