@@ -14,14 +14,50 @@ import h5py
 import xarray as xr
 
 from .types import ComplexNumber, Literal, TYPE_TAG_STR
-from ..log import FileError, log
-from .data.data_array import DataArray, DATA_ARRAY_TAG
+from .data.data_array import DataArray, DATA_ARRAY_MAP
+from ..exceptions import FileError
+from ..log import log
 
 # default indentation (# spaces) in files
 INDENT = 4
 JSON_TAG = "JSON_STRING"
 
 
+def cache(prop):
+    """Decorates a property to cache the first computed value and return it on subsequent calls."""
+
+    # note, we could also just use `prop` as dict key, but hashing property might be slow
+    prop_name = prop.__name__
+
+    @wraps(prop)
+    def cached_property_getter(self):
+        """The new property method to be returned by decorator."""
+
+        stored_value = self._cached_properties.get(prop_name)  # pylint:disable=protected-access
+
+        if stored_value is not None:
+            return stored_value
+
+        computed_value = prop(self)
+        self._cached_properties[prop_name] = computed_value  # pylint:disable=protected-access
+        return computed_value
+
+    return cached_property_getter
+
+
+def cached_property(cached_property_getter):
+    """Shortcut for property(cache()) of a getter."""
+
+    return property(cache(cached_property_getter))
+
+
+def ndarray_encoder(val):
+    """How a ``np.ndarray`` gets handled before saving to json."""
+    if np.any(np.iscomplex(val)):
+        return dict(real=val.real.tolist(), imag=val.imag.tolist())
+    return val.real.tolist()
+
+
 class Tidy3dBaseModel(pydantic.BaseModel):
     """Base pydantic model that all Tidy3d components inherit from.
     Defines configuration for handling data structures
@@ -30,6 +66,14 @@ class Tidy3dBaseModel(pydantic.BaseModel):
     `Pydantic Models <https://pydantic-docs.helpmanual.io/usage/models/>`_
     """
 
+    def __init__(self, **kwargs):
+        """Init method, includes post-init validators."""
+        super().__init__(**kwargs)
+        self._post_init_validators()
+
+    def _post_init_validators(self) -> None:
+        """Call validators taking ``self`` that get run after init, implement in subclasses."""
+
     def __init_subclass__(cls) -> None:
         """Things that are done to each of the models."""
 
@@ -61,7 +105,7 @@ class Tidy3dBaseModel(pydantic.BaseModel):
         validate_assignment = True
         allow_population_by_field_name = True
         json_encoders = {
-            np.ndarray: lambda x: tuple(x.tolist()),
+            np.ndarray: ndarray_encoder,
             complex: lambda x: ComplexNumber(real=x.real, imag=x.imag),
             xr.DataArray: DataArray._json_encoder,  # pylint:disable=unhashable-member, protected-access
         }
@@ -308,7 +352,7 @@ class Tidy3dBaseModel(pydantic.BaseModel):
     @staticmethod
     def _warn_if_contains_data(json_str: str) -> None:
         """Log a warning if the json string contains data, used in '.json' and '.yaml' file."""
-        if DATA_ARRAY_TAG in json_str:
+        if any((key in json_str for key, _ in DATA_ARRAY_MAP.items())):
             log.warning(
                 "Data contents found in the model to be written to file. "
                 "Note that this data will not be included in '.json' or '.yaml' formats. "
@@ -387,8 +431,9 @@ class Tidy3dBaseModel(pydantic.BaseModel):
                 subpath = f"{group_path}/{key}"
 
                 # write the path to the element of the json dict where the data_array should be
-                if value == DATA_ARRAY_TAG:
-                    model_dict[key] = DataArray.from_hdf5(fname=fname, group_path=subpath)
+                if isinstance(value, str) and value in DATA_ARRAY_MAP:
+                    data_array_type = DATA_ARRAY_MAP[value]
+                    model_dict[key] = data_array_type.from_hdf5(fname=fname, group_path=subpath)
                     continue
 
                 # if a list, assign each element a unique key, recurse
@@ -445,32 +490,32 @@ class Tidy3dBaseModel(pydantic.BaseModel):
         >>> simulation.to_hdf5(fname='folder/sim.hdf5') # doctest: +SKIP
         """
 
-        def add_data_to_file(data_dict: dict, group_path: str = "") -> None:
-            """For every DataArray item in dictionary, write path of hdf5 group as value."""
-
-            for key, value in data_dict.items():
-
-                # append the key to the path
-                subpath = f"{group_path}/{key}"
-
-                # write the path to the element of the json dict where the data_array should be
-                if isinstance(value, xr.DataArray):
-                    value.to_hdf5(fname=fname, group_path=subpath)
-
-                # if a tuple, assign each element a unique key
-                if isinstance(value, (list, tuple)):
-                    value_dict = self.tuple_to_dict(tuple_values=value)
-                    add_data_to_file(data_dict=value_dict, group_path=subpath)
-
-                # if a dict, recurse
-                elif isinstance(value, dict):
-                    add_data_to_file(data_dict=value, group_path=subpath)
-
-        json_string = self._json_string
         with h5py.File(fname, "w") as f_handle:
-            f_handle[JSON_TAG] = json_string
 
-        add_data_to_file(data_dict=self.dict())
+            f_handle[JSON_TAG] = self._json_string
+
+            def add_data_to_file(data_dict: dict, group_path: str = "") -> None:
+                """For every DataArray item in dictionary, write path of hdf5 group as value."""
+
+                for key, value in data_dict.items():
+
+                    # append the key to the path
+                    subpath = f"{group_path}/{key}"
+
+                    # write the path to the element of the json dict where the data_array should be
+                    if isinstance(value, xr.DataArray):
+                        value.to_hdf5(fname=f_handle, group_path=subpath)
+
+                    # if a tuple, assign each element a unique key
+                    if isinstance(value, (list, tuple)):
+                        value_dict = self.tuple_to_dict(tuple_values=value)
+                        add_data_to_file(data_dict=value_dict, group_path=subpath)
+
+                    # if a dict, recurse
+                    elif isinstance(value, dict):
+                        add_data_to_file(data_dict=value, group_path=subpath)
+
+            add_data_to_file(data_dict=self.dict())
 
     def __lt__(self, other):
         """define < for getting unique indices based on hash."""
@@ -488,7 +533,11 @@ class Tidy3dBaseModel(pydantic.BaseModel):
         """define >= for getting unique indices based on hash."""
         return hash(self) >= hash(other)
 
-    @property
+    def __eq__(self, other):
+        """Define == for two Tidy3DBaseModels."""
+        return self._json_string == other._json_string
+
+    @cached_property
     def _json_string(self) -> str:
         """Returns string representation of a :class:`Tidy3dBaseModel`.
 
@@ -508,9 +557,10 @@ class Tidy3dBaseModel(pydantic.BaseModel):
 
         json_string = self.json(indent=INDENT, exclude_unset=False)
         json_string = make_json_compatible(json_string)
-        json_dict = json.loads(json_string)
+        return json_string
+        # json_dict = json.loads(json_string)
 
-        return json.dumps(json_dict)
+        # return json.dumps(json_dict)
 
     @classmethod
     def add_type_field(cls) -> None:
@@ -595,31 +645,3 @@ class Tidy3dBaseModel(pydantic.BaseModel):
 
         doc += "\n"
         cls.__doc__ = doc
-
-
-def cache(prop):
-    """Decorates a property to cache the first computed value and return it on subsequent calls."""
-
-    # note, we could also just use `prop` as dict key, but hashing property might be slow
-    prop_name = prop.__name__
-
-    @wraps(prop)
-    def cached_property_getter(self):
-        """The new property method to be returned by decorator."""
-
-        stored_value = self._cached_properties.get(prop_name)  # pylint:disable=protected-access
-
-        if stored_value is not None:
-            return stored_value
-
-        computed_value = prop(self)
-        self._cached_properties[prop_name] = computed_value  # pylint:disable=protected-access
-        return computed_value
-
-    return cached_property_getter
-
-
-def cached_property(cached_property_getter):
-    """Shortcut for property(cache()) of a getter."""
-
-    return property(cache(cached_property_getter))
