@@ -5,6 +5,7 @@ from __future__ import annotations
 from abc import ABC
 from typing import Union, Tuple, Callable, Dict, List
 import warnings
+
 import xarray as xr
 import numpy as np
 import pydantic as pd
@@ -18,7 +19,7 @@ from .data_array import FreqDataArray, TimeDataArray, FreqModeDataArray
 from .dataset import Dataset, AbstractFieldDataset, ElectromagneticFieldDataset
 from .dataset import FieldDataset, FieldTimeDataset, ModeSolverDataset, PermittivityDataset
 from ..base import TYPE_TAG_STR, cached_property
-from ..types import Coordinate, Symmetry, ArrayLike, Size, Numpy, TrackFreq
+from ..types import Coordinate, Symmetry, ArrayFloat1D, Size, Numpy, TrackFreq
 from ..grid.grid import Grid, Coords
 from ..validators import enforce_monitor_fields_present, required_if_symmetry_present
 from ..monitor import MonitorType, FieldMonitor, FieldTimeMonitor, ModeSolverMonitor
@@ -28,8 +29,12 @@ from ..monitor import FieldProjectionKSpaceMonitor, FieldProjectionSurface
 from ..monitor import DiffractionMonitor
 from ..source import SourceTimeType, CustomFieldSource
 from ..medium import Medium, MediumType
-from ...log import SetupError, DataError, Tidy3dNotImplementedError, log
+from ...exceptions import SetupError, DataError, Tidy3dNotImplementedError
 from ...constants import ETA_0, C_0, MICROMETER
+from ...log import log
+
+
+Coords1D = ArrayFloat1D
 
 
 class MonitorData(Dataset, ABC):
@@ -219,7 +224,7 @@ class ElectromagneticFieldData(AbstractFieldData, ElectromagneticFieldDataset, A
         return tan_fields
 
     @property
-    def _plane_grid_boundaries(self) -> Tuple[ArrayLike[float, 1], ArrayLike[float, 1]]:
+    def _plane_grid_boundaries(self) -> Tuple[Coords1D, Coords1D]:
         """For a 2D monitor data, return the boundaries of the in-plane grid to be used to compute
         differential area and to colocate fields to grid centers if needed."""
         if np.any(np.array(self.monitor.interval_space) > 1):
@@ -871,14 +876,14 @@ class FluxTimeData(MonitorData):
     flux: FluxTimeDataArray
 
 
-PROJFIELDTYPE = Union[
+ProjFieldType = Union[
     FieldProjectionAngleDataArray,
     FieldProjectionCartesianDataArray,
     FieldProjectionKSpaceDataArray,
     DiffractionDataArray,
 ]
 
-PROJMONTYPE = Union[
+ProjMonitorType = Union[
     FieldProjectionAngleMonitor,
     FieldProjectionCartesianMonitor,
     FieldProjectionKSpaceMonitor,
@@ -889,38 +894,38 @@ PROJMONTYPE = Union[
 class AbstractFieldProjectionData(MonitorData):
     """Collection of projected fields in spherical coordinates in the frequency domain."""
 
-    monitor: PROJMONTYPE = pd.Field(
+    monitor: ProjMonitorType = pd.Field(
         ...,
         title="Projection monitor",
         description="Field projection monitor.",
     )
 
-    Er: PROJFIELDTYPE = pd.Field(
+    Er: ProjFieldType = pd.Field(
         ...,
         title="Ephi",
         description="Spatial distribution of r-component of the electric field.",
     )
-    Etheta: PROJFIELDTYPE = pd.Field(
+    Etheta: ProjFieldType = pd.Field(
         ...,
         title="Etheta",
         description="Spatial distribution of the theta-component of the electric field.",
     )
-    Ephi: PROJFIELDTYPE = pd.Field(
+    Ephi: ProjFieldType = pd.Field(
         ...,
         title="Ephi",
         description="Spatial distribution of phi-component of the electric field.",
     )
-    Hr: PROJFIELDTYPE = pd.Field(
+    Hr: ProjFieldType = pd.Field(
         ...,
         title="Hphi",
         description="Spatial distribution of r-component of the magnetic field.",
     )
-    Htheta: PROJFIELDTYPE = pd.Field(
+    Htheta: ProjFieldType = pd.Field(
         ...,
         title="Htheta",
         description="Spatial distribution of theta-component of the magnetic field.",
     )
-    Hphi: PROJFIELDTYPE = pd.Field(
+    Hphi: ProjFieldType = pd.Field(
         ...,
         title="Hphi",
         description="Spatial distribution of phi-component of the magnetic field.",
