@@ -10,13 +10,13 @@ import pydantic as pd
 from .grid import Coords1D, Coords, Grid
 from .mesher import GradedMesher, MesherType
 from ..base import Tidy3dBaseModel
-from ..types import Axis
-from ..boundary import Symmetry
+from ..types import Axis, Symmetry, annotate_type
 from ..source import SourceType
-from ..structure import Structure
+from ..structure import Structure, StructureType
 from ..geometry import Box
-from ...log import SetupError, log
-from ...constants import C_0, MICROMETER
+from ...log import log
+from ...exceptions import SetupError
+from ...constants import MICROMETER, C_0, fp_eps
 
 
 class GridSpec1d(Tidy3dBaseModel, ABC):
@@ -26,8 +26,9 @@ class GridSpec1d(Tidy3dBaseModel, ABC):
     def make_coords(  # pylint:disable = too-many-arguments
         self,
         axis: Axis,
-        structures: List[Structure],
+        structures: List[StructureType],
         symmetry: Tuple[Symmetry, Symmetry, Symmetry],
+        periodic: bool,
         wavelength: pd.PositiveFloat,
         num_pml_layers: Tuple[pd.NonNegativeInt, pd.NonNegativeInt],
     ) -> Coords1D:
@@ -38,7 +39,7 @@ class GridSpec1d(Tidy3dBaseModel, ABC):
         ----------
         axis : Axis
             Axis of this direction.
-        structures : List[Structure]
+        structures : List[StructureType]
             List of structures present in simulation, the first one being the simulation domain.
         symmetry : Tuple[Symmetry, Symmetry, Symmetry]
             Reflection symmetry across a plane bisecting the simulation domain
@@ -56,7 +57,7 @@ class GridSpec1d(Tidy3dBaseModel, ABC):
 
         # Determine if one should apply periodic boundary condition.
         # This should only affect auto nonuniform mesh generation for now.
-        is_periodic = sum(num_pml_layers) == 0 and symmetry[axis] == 0
+        is_periodic = periodic and symmetry[axis] == 0
 
         # generate boundaries
         bound_coords = self._make_coords_initial(
@@ -84,7 +85,7 @@ class GridSpec1d(Tidy3dBaseModel, ABC):
     def _make_coords_initial(
         self,
         axis: Axis,
-        structures: List[Structure],
+        structures: List[StructureType],
         **kwargs,
     ) -> Coords1D:
         """Generate 1D coords to be used as grid boundaries, based on simulation parameters.
@@ -94,7 +95,7 @@ class GridSpec1d(Tidy3dBaseModel, ABC):
 
         Parameters
         ----------
-        structures : List[Structure]
+        structures : List[StructureType]
             List of structures present in simulation, the first one being the simulation domain.
         **kwargs
             Other arguments
@@ -151,7 +152,7 @@ class UniformGrid(GridSpec1d):
     def _make_coords_initial(
         self,
         axis: Axis,
-        structures: List[Structure],
+        structures: List[StructureType],
         **kwargs,
     ) -> Coords1D:
         """Uniform 1D coords to be used as grid boundaries.
@@ -160,7 +161,7 @@ class UniformGrid(GridSpec1d):
         ----------
         axis : Axis
             Axis of this direction.
-        structures : List[Structure]
+        structures : List[StructureType]
             List of structures present in simulation, the first one being the simulation domain.
         **kwargs:
             Other arguments all go here.
@@ -199,16 +200,25 @@ class CustomGrid(GridSpec1d):
         title="Customized grid sizes.",
         description="An array of custom nonuniform grid sizes. The resulting grid is centered on "
         "the simulation center such that it spans the region "
-        "``(center - sum(dl)/2, center + sum(dl)/2)``. "
+        "``(center - sum(dl)/2, center + sum(dl)/2)``, unless a ``custom_offset`` is given. "
         "Note: if supplied sizes do not cover the simulation size, the first and last sizes "
         "are repeated to cover the simulation domain.",
         units=MICROMETER,
     )
 
+    custom_offset: float = pd.Field(
+        None,
+        title="Customized grid offset.",
+        description="The starting coordinate of the grid which defines the simulation center. "
+        "If ``None``, the simulation center is set such that it spans the region "
+        "``(center - sum(dl)/2, center + sum(dl)/2)``.",
+        units=MICROMETER,
+    )
+
     def _make_coords_initial(
         self,
         axis: Axis,
-        structures: List[Structure],
+        structures: List[StructureType],
         **kwargs,
     ) -> Coords1D:
         """Customized 1D coords to be used as grid boundaries.
@@ -217,7 +227,7 @@ class CustomGrid(GridSpec1d):
         ----------
         axis : Axis
             Axis of this direction.
-        structures : List[Structure]
+        structures : List[StructureType]
             List of structures present in simulation, the first one being the simulation domain.
         *kwargs
             Other arguments all go here.
@@ -234,12 +244,18 @@ class CustomGrid(GridSpec1d):
         dl = np.array(self.dl)
         bound_coords = np.append(0.0, np.cumsum(dl))
 
-        # place the middle of the bounds at the center of the simulation along dimension
-        bound_coords += center - bound_coords[-1] / 2
+        # place the middle of the bounds at the center of the simulation along dimension,
+        # or use the `custom_offset` if provided
+        if self.custom_offset is None:
+            bound_coords += center - bound_coords[-1] / 2
+        else:
+            bound_coords += self.custom_offset
 
-        # chop off any coords outside of simulation bounds
-        bound_min = center - size / 2
-        bound_max = center + size / 2
+        # chop off any coords outside of simulation bounds, beyond some buffer region
+        # to take numerical effects into account
+        buffer = fp_eps * size
+        bound_min = center - size / 2 - buffer
+        bound_max = center + size / 2 + buffer
         bound_coords = bound_coords[bound_coords <= bound_max]
         bound_coords = bound_coords[bound_coords >= bound_min]
 
@@ -251,6 +267,14 @@ class CustomGrid(GridSpec1d):
         while bound_coords[-1] + dl_max <= bound_max:
             bound_coords = np.append(bound_coords, bound_coords[-1] + dl_max)
 
+        # in case a `custom_offset` is provided, it's possible the bounds were numerically within
+        # the simulation bounds but were still chopped off, which is fixed here
+        if self.custom_offset is not None:
+            if np.isclose(bound_coords[0] - dl_min, bound_min):
+                bound_coords = np.insert(bound_coords, 0, bound_coords[0] - dl_min)
+            if np.isclose(bound_coords[-1] + dl_max, bound_max):
+                bound_coords = np.append(bound_coords, bound_coords[-1] + dl_max)
+
         return bound_coords
 
 
@@ -277,6 +301,15 @@ class AutoGrid(GridSpec1d):
         lt=2.0,
     )
 
+    dl_min: pd.NonNegativeFloat = pd.Field(
+        0,
+        title="Lower bound of grid size",
+        description="Lower bound of the grid size along this dimension regardless of "
+        "structures present in the simulation, including override structures "
+        "with ``enforced=True``. It is a soft bound, meaning that the actual minimal "
+        "grid size might be slightly smaller.",
+    )
+
     mesher: MesherType = pd.Field(
         GradedMesher(),
         title="Grid Construction Tool",
@@ -286,7 +319,7 @@ class AutoGrid(GridSpec1d):
     def _make_coords_initial(  # pylint:disable=too-many-arguments,arguments-differ,too-many-locals
         self,
         axis: Axis,
-        structures: List[Structure],
+        structures: List[StructureType],
         wavelength: float,
         symmetry: Symmetry,
         is_periodic: bool,
@@ -297,7 +330,7 @@ class AutoGrid(GridSpec1d):
         ----------
         axis : Axis
             Axis of this direction.
-        structures : List[Structure]
+        structures : List[StructureType]
             List of structures present in simulation.
         wavelength : float
             Free-space wavelength.
@@ -329,7 +362,11 @@ class AutoGrid(GridSpec1d):
 
         # parse structures
         interval_coords, max_dl_list = self.mesher.parse_structures(
-            axis, struct_list, wavelength, self.min_steps_per_wvl
+            axis,
+            struct_list,
+            wavelength,
+            self.min_steps_per_wvl,
+            self.dl_min,
         )
         # Put just a single pixel if 2D-like simulation
         if interval_coords.size == 1:
@@ -400,16 +437,20 @@ class GridSpec(Tidy3dBaseModel):
         title="Free-space wavelength",
         description="Free-space wavelength for automatic nonuniform grid. It can be 'None' "
         "if there is at least one source in the simulation, in which case it is defined by "
-        "the source central frequency.",
+        "the source central frequency. "
+        "Note: it only takes effect when at least one of the three dimensions "
+        "uses :class:`.AutoGrid`.",
         units=MICROMETER,
     )
 
-    override_structures: Tuple[Structure, ...] = pd.Field(
+    override_structures: Tuple[annotate_type(StructureType), ...] = pd.Field(
         (),
         title="Grid specification override structures",
         description="A set of structures that is added on top of the simulation structures in "
         "the process of generating the grid. This can be used to refine the grid or make it "
-        "coarser depending than the expected need for higher/lower resolution regions.",
+        "coarser depending than the expected need for higher/lower resolution regions. "
+        "Note: it only takes effect when at least one of the three dimensions "
+        "uses :class:`.AutoGrid`.",
     )
 
     @property
@@ -418,19 +459,28 @@ class GridSpec(Tidy3dBaseModel):
         grid_list = [self.grid_x, self.grid_y, self.grid_z]
         return np.any([isinstance(mesh, AutoGrid) for mesh in grid_list])
 
-    def wavelength_from_sources(self, sources: List[SourceType]) -> pd.PositiveFloat:
+    @property
+    def custom_grid_used(self) -> bool:
+        """True if any of the three dimensions uses :class:`.CustomGrid`."""
+        grid_list = [self.grid_x, self.grid_y, self.grid_z]
+        return np.any([isinstance(mesh, CustomGrid) for mesh in grid_list])
+
+    @staticmethod
+    def wavelength_from_sources(sources: List[SourceType]) -> pd.PositiveFloat:
         """Define a wavelength based on supplied sources. Called if auto mesh is used and
         ``self.wavelength is None``."""
 
-        # Use central frequency of sources, if any.
-        freqs = np.array([source.source_time.freq0 for source in sources])
         # no sources
-        if len(freqs) == 0:
+        if len(sources) == 0:
             raise SetupError(
                 "Automatic grid generation requires the input of 'wavelength' or sources."
             )
+
+        # Use central frequency of sources, if any.
+        freqs = np.array([source.source_time.freq0 for source in sources])
+
         # multiple sources of different central frequencies
-        if len(freqs) > 0 and not np.all(np.isclose(freqs, freqs[0])):
+        if not np.all(np.isclose(freqs, freqs[0])):
             raise SetupError(
                 "Sources of different central frequencies are supplied. "
                 "Please supply a 'wavelength' value for 'grid_spec'."
@@ -438,10 +488,34 @@ class GridSpec(Tidy3dBaseModel):
 
         return C_0 / freqs[0]
 
+    @property
+    def override_structures_used(self) -> List[bool, bool, bool]:
+        """Along each axis, ``True`` if any override structure is used. However,
+        it is still ``False`` if only :class:`.MeshOverrideStructure` is supplied, and
+        their ``dl[axis]`` all take the ``None`` value.
+        """
+
+        # empty override_structure list
+        if len(self.override_structures) == 0:
+            return [False] * 3
+
+        override_used = [False] * 3
+        for structure in self.override_structures:
+            # override used in all axes if any `Structure` is present
+            if isinstance(structure, Structure):
+                return [True] * 3
+
+            for dl_axis, dl in enumerate(structure.dl):
+                if (not override_used[dl_axis]) and (dl is not None):
+                    override_used[dl_axis] = True
+        return override_used
+
+    # pylint:disable=too-many-locals,too-many-arguments
     def make_grid(
         self,
         structures: List[Structure],
         symmetry: Tuple[Symmetry, Symmetry, Symmetry],
+        periodic: Tuple[bool, bool, bool],
         sources: List[SourceType],
         num_pml_layers: List[Tuple[pd.NonNegativeInt, pd.NonNegativeInt]],
     ) -> Grid:
@@ -468,33 +542,36 @@ class GridSpec(Tidy3dBaseModel):
 
         # Set up wavelength for automatic mesh generation if needed.
         wavelength = self.wavelength
-        if self.wavelength is None and self.auto_grid_used:
+        if wavelength is None and self.auto_grid_used:
             wavelength = self.wavelength_from_sources(sources)
             log.info(f"Auto meshing using wavelength {wavelength:1.4f} defined from sources.")
 
-        coords_x = self.grid_x.make_coords(
-            axis=0,
-            structures=list(structures) + list(self.override_structures),
-            symmetry=symmetry,
-            wavelength=wavelength,
-            num_pml_layers=num_pml_layers[0],
-        )
-        coords_y = self.grid_y.make_coords(
-            axis=1,
-            structures=list(structures) + list(self.override_structures),
-            symmetry=symmetry,
-            wavelength=wavelength,
-            num_pml_layers=num_pml_layers[1],
-        )
-        coords_z = self.grid_z.make_coords(
-            axis=2,
-            structures=list(structures) + list(self.override_structures),
-            symmetry=symmetry,
-            wavelength=wavelength,
-            num_pml_layers=num_pml_layers[2],
-        )
+        # Warn user if ``GridType`` along some axis is not ``AutoGrid`` and
+        # ``override_structures`` is not empty. The override structures
+        # are not effective along those axes.
+        for axis_ind, override_used_axis, grid_axis in zip(
+            ["x", "y", "z"], self.override_structures_used, [self.grid_x, self.grid_y, self.grid_z]
+        ):
+            if override_used_axis and not isinstance(grid_axis, AutoGrid):
+                log.warning(
+                    f"Override structures take no effect along {axis_ind}-axis. "
+                    "If intending to apply override structures to this axis, "
+                    "use 'AutoGrid'."
+                )
 
-        coords = Coords(x=coords_x, y=coords_y, z=coords_z)
+        grids_1d = [self.grid_x, self.grid_y, self.grid_z]
+        coords_dict = {}
+        for idim, (dim, grid_1d) in enumerate(zip("xyz", grids_1d)):
+            coords_dict[dim] = grid_1d.make_coords(
+                axis=idim,
+                structures=list(structures) + list(self.override_structures),
+                symmetry=symmetry,
+                periodic=periodic[idim],
+                wavelength=wavelength,
+                num_pml_layers=num_pml_layers[idim],
+            )
+
+        coords = Coords(**coords_dict)
         return Grid(boundaries=coords)
 
     @classmethod
@@ -503,7 +580,8 @@ class GridSpec(Tidy3dBaseModel):
         wavelength: pd.PositiveFloat = None,
         min_steps_per_wvl: pd.PositiveFloat = 10.0,
         max_scale: pd.PositiveFloat = 1.4,
-        override_structures: List[Structure] = (),
+        override_structures: List[StructureType] = (),
+        dl_min: pd.NonNegativeFloat = 0.0,
         mesher: MesherType = GradedMesher(),
     ) -> GridSpec:
         """Use the same :class:`AutoGrid` along each of the three directions.
@@ -518,10 +596,12 @@ class GridSpec(Tidy3dBaseModel):
             Minimal number of steps per wavelength in each medium.
         max_scale : pd.PositiveFloat, optional
             Sets the maximum ratio between any two consecutive grid steps.
-        override_structures : List[Structure]
+        override_structures : List[StructureType]
             A list of structures that is added on top of the simulation structures in
             the process of generating the grid. This can be used to refine the grid or make it
             coarser depending than the expected need for higher/lower resolution regions.
+        dl_min: pd.NonNegativeFloat
+            Lower bound of grid size.
         mesher : MesherType = GradedMesher()
             The type of mesher to use to generate the grid automatically.
 
@@ -531,7 +611,12 @@ class GridSpec(Tidy3dBaseModel):
             :class:`GridSpec` with the same automatic nonuniform grid settings in each direction.
         """
 
-        grid_1d = AutoGrid(min_steps_per_wvl=min_steps_per_wvl, max_scale=max_scale, mesher=mesher)
+        grid_1d = AutoGrid(
+            min_steps_per_wvl=min_steps_per_wvl,
+            max_scale=max_scale,
+            dl_min=dl_min,
+            mesher=mesher,
+        )
         return cls(
             wavelength=wavelength,
             grid_x=grid_1d,
