@@ -4,41 +4,48 @@
 from abc import ABC, abstractmethod
 from typing import Tuple, List, Union, Dict
 from math import isclose
+from itertools import compress
+import warnings
 
 import pydantic as pd
 import numpy as np
 from pyroots import Brentq
 from shapely.strtree import STRtree
 from shapely.geometry import box as shapely_box
+from shapely.errors import ShapelyDeprecationWarning
 
 from ..base import Tidy3dBaseModel
-from ..types import Axis, ArrayLike
-from ..structure import Structure
-from ...log import SetupError, ValidationError
+from ..types import Axis, ArrayFloat1D
+from ..structure import Structure, MeshOverrideStructure, StructureType
+from ..medium import PECMedium, Medium2D
+from ...exceptions import SetupError, ValidationError
 from ...constants import C_0, fp_eps
 
+_ROOTS_TOL = 1e-10
+
 
 class Mesher(Tidy3dBaseModel, ABC):
     """Abstract class for automatic meshing."""
 
     @abstractmethod
-    def parse_structures(
+    def parse_structures(  # pylint:disable=too-many-arguments
         self,
         axis: Axis,
-        structures: List[Structure],
+        structures: List[StructureType],
         wavelength: pd.PositiveFloat,
         min_steps_per_wvl: pd.NonNegativeInt,
-    ) -> Tuple[ArrayLike[float, 1], ArrayLike[float, 1]]:
+        dl_min: pd.NonNegativeFloat,
+    ) -> Tuple[ArrayFloat1D, ArrayFloat1D]:
         """Calculate the positions of all bounding box interfaces along a given axis."""
 
     @abstractmethod
     def make_grid_multiple_intervals(
         self,
-        max_dl_list: ArrayLike[float, 1],
-        len_interval_list: ArrayLike[float, 1],
+        max_dl_list: ArrayFloat1D,
+        len_interval_list: ArrayFloat1D,
         max_scale: float,
         is_periodic: bool,
-    ) -> List[ArrayLike[float, 1]]:
+    ) -> List[ArrayFloat1D]:
         """Create grid steps in multiple connecting intervals."""
 
 
@@ -46,14 +53,15 @@ class GradedMesher(Mesher):
     """Implements automatic nonuniform meshing with a set minimum steps per wavelength and
     a graded mesh expanding from higher- to lower-resolution regions."""
 
-    # pylint:disable=too-many-locals
+    # pylint:disable=too-many-locals, too-many-arguments
     def parse_structures(
         self,
         axis: Axis,
-        structures: List[Structure],
+        structures: List[StructureType],
         wavelength: pd.PositiveFloat,
         min_steps_per_wvl: pd.NonNegativeInt,
-    ) -> Tuple[ArrayLike[float, 1], ArrayLike[float, 1]]:
+        dl_min: pd.NonNegativeFloat,
+    ) -> Tuple[ArrayFloat1D, ArrayFloat1D]:
         """Calculate the positions of all bounding box interfaces along a given axis.
         In this implementation, in most cases the complexity should be O(len(structures)**2),
         although the worst-case complexity may approach O(len(structures)**3).
@@ -63,12 +71,14 @@ class GradedMesher(Mesher):
         ----------
         axis : Axis
             Axis index along which to operate.
-        structures : List[Structure]
+        structures : List[StructureType]
             List of structures, with the simulation structure being the first item.
         wavelength : pd.PositiveFloat
             Wavelength to use for the step size and for dispersive media epsilon.
         min_steps_per_wvl : pd.NonNegativeInt
             Minimum requested steps per wavelength.
+        dl_min: pd.NonNegativeFloat
+            Lower bound of grid size.
 
         Returns
         -------
@@ -88,17 +98,41 @@ class GradedMesher(Mesher):
         sim_bmin, sim_bmax = structures[0].geometry.bounds
         domain_bounds = np.array([sim_bmin[axis], sim_bmax[axis]])
 
+        # For MeshOverrideStructure, we allow ``dl`` along some axis to be ``None``
+        # so that no override occurs along this axis. So first, we filter all those
+        # structures.
+        structures_effective = self.filter_structures_effective_dl(structures, axis)
+
+        # Special attention needs to be paid to enforced overrideStructures.
+        # They shouldn't be overridden by other structures;
+        # for overlapping enforced structures, the grid size of the overlapped
+        # region is determined by the last enforced structure. We take two
+        # steps to implement the feature:
+        # 1) reorder structure list so that enforce = True structures are shifted to
+        #    the end of the last.
+        # 2) in each interval, the maximal grid size is:
+        #    a) no enforced structure: min(grid size of each structure).
+        #    b) with enforced structure: grid size of the last override structure.
+
+        # reorder structure list to place enforced ones to the end
+        num_unenforced, structures_ordered = self.reorder_structures_enforced_to_end(
+            structures_effective
+        )
+
         # Required maximum steps in every structure
-        structure_steps = self.structure_steps(structures, wavelength, min_steps_per_wvl)
+        structure_steps = self.structure_steps(
+            structures_ordered, wavelength, min_steps_per_wvl, dl_min, axis
+        )
         # Smallest of the maximum steps
         min_step = np.amin(structure_steps)
 
         # If empty simulation, return
         if len(structures) == 1:
-            return (domain_bounds, structure_steps)
+            interval_coords, max_steps = self.filter_min_step(domain_bounds, structure_steps)
+            return np.array(interval_coords), np.array(max_steps)
 
         # Bounding boxes with the meshing axis rotated to z
-        struct_bbox = self.rotate_structure_bounds(structures, axis)
+        struct_bbox = self.rotate_structure_bounds(structures_ordered, axis)
         # Rtree from the 2D part of the bounding boxes
         tree = self.bounds_2d_tree(struct_bbox)
 
@@ -107,7 +141,7 @@ class GradedMesher(Mesher):
         # containment then we need to populate interval coordinates starting from the top.
         # If a structure is found to be completely contained, the corresponding ``struct_bbox`` is
         # set to ``None``.
-        for str_ind in range(len(structures) - 1, -1, -1):
+        for str_ind in range(len(structures_ordered) - 1, -1, -1):
             # 3D and 2D bounding box of current structure
             bbox = struct_bbox[str_ind]
             if bbox is None:
@@ -116,7 +150,10 @@ class GradedMesher(Mesher):
             bbox_2d = shapely_box(bbox[0, 0], bbox[0, 1], bbox[1, 0], bbox[1, 1])
 
             # List of structure indexes that may intersect the current structure in 2D
-            query_inds = tree.query_items(bbox_2d)
+            try:
+                query_inds = tree.query_items(bbox_2d)
+            except AttributeError:
+                query_inds = tree.query(bbox_2d)
 
             # Remove all lower structures that the current structure completely contains
             inds_lower = [
@@ -145,9 +182,15 @@ class GradedMesher(Mesher):
         # Compute the maximum allowed step size in each interval
         max_steps = []
         for coord_ind, _ in enumerate(intervals["coords"][:-1]):
-            # Define the max step as the minimum over all medium steps of media in this interval
-            max_step = np.amin(structure_steps[intervals["structs"][coord_ind]])
-            max_steps.append(float(max_step))
+            # if there are any enforced structure in the interval, use the last structure
+            if max(intervals["structs"][coord_ind]) >= num_unenforced:
+                max_step = structure_steps[max(intervals["structs"][coord_ind])]
+                max_steps.append(float(max_step))
+            # otherwise, define the max step as the minimum over all medium steps
+            # of media in this interval
+            else:
+                max_step = np.amin(structure_steps[intervals["structs"][coord_ind]])
+                max_steps.append(float(max_step))
 
         # Re-evaluate the absolute smallest min_step and remove intervals that are smaller than that
         intervals["coords"], max_steps = self.filter_min_step(intervals["coords"], max_steps)
@@ -159,8 +202,8 @@ class GradedMesher(Mesher):
         self,
         intervals: Dict[str, List],
         str_ind: int,
-        str_bbox: ArrayLike[float, 1],
-        bbox_contained_2d: List[ArrayLike[float, 1]],
+        str_bbox: ArrayFloat1D,
+        bbox_contained_2d: List[ArrayFloat1D],
         min_step: float,
     ) -> Dict[str, List]:
         """Figure out where to place the bounding box coordinates of current structure.
@@ -180,9 +223,9 @@ class GradedMesher(Mesher):
             of lists of structures contained in each interval.
         str_ind : int
             Index of the current structure.
-        str_bbox : ArrayLike[float, 1]
+        str_bbox : ArrayFloat1D
             Bounding box of the current structure.
-        bbox_contained_2d : List[ArrayLike[float, 1]]
+        bbox_contained_2d : List[ArrayFloat1D]
             List of 3D bounding boxes that contain the current structure in 2D.
         min_step : float
             Absolute minimum interval size to impose.
@@ -240,11 +283,82 @@ class GradedMesher(Mesher):
 
         return {"coords": coords, "structs": structs}
 
+    @staticmethod
+    def reorder_structures_enforced_to_end(
+        structures: List[StructureType],
+    ) -> Tuple[int, List[StructureType]]:
+        """Reorder structure list so that MeshOverrideStructures with ``enforce=True``
+        are shifted to the end of list.
+
+        Parameters
+        ----------
+        structures : List[StructureType]
+            List of structures, with the simulation structure being the first item.
+
+        Returns
+        -------
+        Tuple[int, List[StructureType]]
+            The number of unenforced structures, reordered structure list
+
+        """
+
+        # boolean list for enforced unenforced structures
+        enforced_list = [
+            isinstance(structure, MeshOverrideStructure) and structure.enforce
+            for structure in structures
+        ]
+
+        # if no enforced structure, a quick return here
+        if not any(enforced_list):
+            return len(structures), structures
+
+        # filter structures
+        structures_enforced = list(compress(structures, enforced_list))
+        structures_others = list(compress(structures, [not enforced for enforced in enforced_list]))
+
+        return len(structures_others), structures_others + structures_enforced
+
+    @staticmethod
+    def filter_structures_effective_dl(
+        structures: List[StructureType], axis: Axis
+    ) -> List[StructureType]:
+        """For :class:`.MeshOverrideStructure`, we allow ``dl`` along some axis
+        to be ``None`` so that no override occurs along this axis.Here those
+        structures with ``dl[axis]=None`` is filtered.
+
+        Parameters
+        ----------
+        structures : List[StructureType]
+            List of structures, with the simulation structure being the first item.
+        axis : Axis
+            Axis index to place last.
+
+        Returns
+        -------
+        List[StructureType]
+            A list of filtered structures whose ``dl`` along this axis is not ``None``.
+        """
+
+        structures_filtered = []
+        for structure in structures:
+            if isinstance(structure, MeshOverrideStructure):
+                # skip override structure if dl not defined along axis
+                if structure.dl[axis] is None:
+                    continue
+            structures_filtered.append(structure)
+        return structures_filtered
+
     @staticmethod
     def structure_steps(
-        structures: List[Structure], wavelength: float, min_steps_per_wvl: float
-    ) -> ArrayLike[float, 1]:
-        """Get the minimum mesh required in each structure.
+        structures: List[StructureType],
+        wavelength: float,
+        min_steps_per_wvl: float,
+        dl_min: pd.NonNegativeFloat,
+        axis: Axis,
+    ) -> ArrayFloat1D:
+        """Get the minimum mesh required in each structure. Special media are set to index of 1,
+        which would usually make the medium ignored in the meshing, while the structure geometry
+        would still be used to place grid boundaries.
 
         Parameters
         ----------
@@ -254,30 +368,40 @@ class GradedMesher(Mesher):
             Wavelength to use for the step size and for dispersive media epsilon.
         min_steps_per_wvl : float
             Minimum requested steps per wavelength.
+        dl_min: pd.NonNegativeFloat
+            Lower bound of grid size.
+        axis : Axis
+            Axis index along which to operate.
         """
         min_steps = []
         for structure in structures:
-            n, k = structure.medium.eps_complex_to_nk(structure.medium.eps_model(C_0 / wavelength))
-            index = max(abs(n), abs(k))
-            min_steps.append(wavelength / index / min_steps_per_wvl)
+            if isinstance(structure, Structure):
+                if isinstance(structure.medium, (PECMedium, Medium2D)):
+                    index = 1.0
+                else:
+                    n, k = structure.medium.eps_complex_to_nk(
+                        structure.medium.eps_diagonal(C_0 / wavelength)[axis]
+                    )
+                    index = max(abs(n), abs(k))
+                min_steps.append(max(dl_min, wavelength / index / min_steps_per_wvl))
+            elif isinstance(structure, MeshOverrideStructure):
+                min_steps.append(max(dl_min, structure.dl[axis]))
         return np.array(min_steps)
 
     @staticmethod
-    def rotate_structure_bounds(
-        structures: List[Structure], axis: Axis
-    ) -> List[ArrayLike[float, 1]]:
+    def rotate_structure_bounds(structures: List[StructureType], axis: Axis) -> List[ArrayFloat1D]:
         """Get sturcture bounding boxes with a given ``axis`` rotated to z.
 
         Parameters
         ----------
-        structures : List[Structure]
+        structures : List[StructureType]
             List of structures, with the simulation structure being the first item.
         axis : Axis
             Axis index to place last.
 
         Returns
         -------
-        List[ArrayLike[float, 1]]
+        List[ArrayFloat1D]
             A list of the bounding boxes of shape ``(2, 3)`` for each structure, with the bounds
             along ``axis`` being ``(:, 2)``.
         """
@@ -292,7 +416,7 @@ class GradedMesher(Mesher):
         return struct_bbox
 
     @staticmethod
-    def bounds_2d_tree(struct_bbox: List[ArrayLike[float, 1]]):
+    def bounds_2d_tree(struct_bbox: List[ArrayFloat1D]):
         """Make a shapely Rtree for the 2D bounding boxes of all structures in the plane
         perpendicular to the meshing axis."""
 
@@ -300,12 +424,14 @@ class GradedMesher(Mesher):
         for bbox in struct_bbox:
             box = shapely_box(bbox[0, 0], bbox[0, 1], bbox[1, 0], bbox[1, 1])
             boxes_2d.append(box)
-        return STRtree(boxes_2d)
+
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", category=ShapelyDeprecationWarning)
+            stree = STRtree(boxes_2d)
+        return stree
 
     @staticmethod
-    def contained_2d(
-        bbox0: ArrayLike[float, 1], query_bbox: List[ArrayLike[float, 1]]
-    ) -> List[ArrayLike[float, 1]]:
+    def contained_2d(bbox0: ArrayFloat1D, query_bbox: List[ArrayFloat1D]) -> List[ArrayFloat1D]:
         """Return a list of all bounding boxes among ``query_bbox`` that contain ``bbox0`` in 2D."""
         return [
             bbox
@@ -321,7 +447,7 @@ class GradedMesher(Mesher):
         ]
 
     @staticmethod
-    def contains_3d(bbox0: ArrayLike[float, 1], query_bbox: List[ArrayLike[float, 1]]) -> List[int]:
+    def contains_3d(bbox0: ArrayFloat1D, query_bbox: List[ArrayFloat1D]) -> List[int]:
         """Return a list of all indexes of bounding boxes in the ``query_bbox`` list that ``bbox0``
         fully contains."""
         return [
@@ -350,7 +476,7 @@ class GradedMesher(Mesher):
         )
 
     @staticmethod
-    def is_contained(normal_pos: float, contained_2d: List[ArrayLike[float, 1]]) -> bool:
+    def is_contained(normal_pos: float, contained_2d: List[ArrayFloat1D]) -> bool:
         """Check if a given ``normal_pos`` along the meshing direction is contained inside any
         of the bounding boxes that are in the ``contained_2d`` list.
         """
@@ -379,20 +505,20 @@ class GradedMesher(Mesher):
 
     def make_grid_multiple_intervals(  # pylint:disable=too-many-locals
         self,
-        max_dl_list: ArrayLike[float, 1],
-        len_interval_list: ArrayLike[float, 1],
+        max_dl_list: ArrayFloat1D,
+        len_interval_list: ArrayFloat1D,
         max_scale: float,
         is_periodic: bool,
-    ) -> List[ArrayLike[float, 1]]:
+    ) -> List[ArrayFloat1D]:
         """Create grid steps in multiple connecting intervals of length specified by
         ``len_interval_list``. The maximal allowed step size in each interval is given by
         ``max_dl_list``. The maximum ratio between neighboring steps is bounded by ``max_scale``.
 
         Parameters
         ----------
-        max_dl_list : ArrayLike[float, 1]
+        max_dl_list : ArrayFloat1D
             Maximal allowed step size of each interval.
-        len_interval_list : ArrayLike[float, 1]
+        len_interval_list : ArrayFloat1D
             A list of interval lengths
         max_scale : float
             Maximal ratio between consecutive steps.
@@ -401,7 +527,7 @@ class GradedMesher(Mesher):
 
         Returns
         -------
-        List[ArrayLike[float, 1]]
+        List[ArrayFloat1D]
             A list of of step sizes in each interval.
         """
 
@@ -475,19 +601,19 @@ class GradedMesher(Mesher):
 
     def grid_multiple_interval_analy_refinement(
         self,
-        max_dl_list: ArrayLike[float, 1],
-        len_interval_list: ArrayLike[float, 1],
+        max_dl_list: ArrayFloat1D,
+        len_interval_list: ArrayFloat1D,
         max_scale: float,
         is_periodic: bool,
-    ) -> Tuple[ArrayLike[float, 1], ArrayLike[float, 1]]:
+    ) -> Tuple[ArrayFloat1D, ArrayFloat1D]:
         """Analytical refinement for multiple intervals. "analytical" meaning we allow
         non-integar step sizes, so that we don't consider snapping here.
 
         Parameters
         ----------
-        max_dl_list : ArrayLike[float, 1]
+        max_dl_list : ArrayFloat1D
             Maximal allowed step size of each interval.
-        len_interval_list : ArrayLike[float, 1]
+        len_interval_list : ArrayFloat1D
             A list of interval lengths
         max_scale : float
             Maximal ratio between consecutive steps.
@@ -496,7 +622,7 @@ class GradedMesher(Mesher):
 
         Returns
         -------
-        Tuple[ArrayLike[float, 1], ArrayLike[float, 1]]
+        Tuple[ArrayFloat1D, ArrayFloat1D]
             left and right step sizes of each interval.
         """
 
@@ -564,7 +690,7 @@ class GradedMesher(Mesher):
         max_dl: float,
         max_scale: float,
         len_interval: float,
-    ) -> ArrayLike[float, 1]:
+    ) -> ArrayFloat1D:
         """Create a set of grid steps in an interval of length ``len_interval``,
         with first step no larger than ``max_scale * left_neighbor_dl`` and last step no larger than
         ``max_scale * right_neighbor_dl``, with maximum ratio ``max_scale`` between
@@ -585,7 +711,7 @@ class GradedMesher(Mesher):
 
         Returns
         -------
-        ArrayLike[float, 1]
+        ArrayFloat1D
             A list of step sizes in the interval.
         """
 
@@ -670,7 +796,7 @@ class GradedMesher(Mesher):
         max_dl: float,
         max_scale: float,
         len_interval: float,
-    ) -> ArrayLike[float, 1]:
+    ) -> ArrayFloat1D:
         """In an interval, grid grows, plateau, and decrease, resembling Lambda letter but
         with plateau in the connection part..
 
@@ -739,7 +865,7 @@ class GradedMesher(Mesher):
         right_dl: float,
         max_scale: float,
         len_interval: float,
-    ) -> ArrayLike[float, 1]:
+    ) -> ArrayFloat1D:
         """In an interval, grid grows, and decrease, resembling Lambda letter.
 
         Parameters
@@ -827,7 +953,7 @@ class GradedMesher(Mesher):
         large_dl: float,
         max_scale: float,
         len_interval: float,
-    ) -> ArrayLike[float, 1]:
+    ) -> ArrayFloat1D:
         """In an interval, grid grows, then plateau.
 
         Parameters
@@ -843,7 +969,7 @@ class GradedMesher(Mesher):
 
         Returns
         -------
-        ArrayLike[float, 1]
+        ArrayFloat1D
             A list of step sizes in the interval, in ascending order.
         """
         # steps for scaling
@@ -881,7 +1007,7 @@ class GradedMesher(Mesher):
         small_dl: float,
         max_scale: float,
         len_interval: float,
-    ) -> ArrayLike[float, 1]:
+    ) -> ArrayFloat1D:
         """Mesh simply grows in an interval.
 
         Parameters
@@ -895,7 +1021,7 @@ class GradedMesher(Mesher):
 
         Returns
         -------
-        ArrayLike[float, 1]
+        ArrayFloat1D
             A list of step sizes in the interval, in ascending order.
         """
 
@@ -947,11 +1073,11 @@ class GradedMesher(Mesher):
 
             # solve for new scaling factor
             # let's not raise exception here, but manually check the convergence.
-            root_scalar = Brentq(raise_on_fail=False, epsilon=fp_eps)
+            root_scalar = Brentq(raise_on_fail=False, epsilon=_ROOTS_TOL)
             sol_scale = root_scalar(fun_scale, 1, max_scale)
 
             # convergence check based on pyroots API and manual evaluation of the function.
-            if sol_scale.converged and abs(fun_scale(sol_scale.x0)) <= fp_eps:
+            if sol_scale.converged and abs(fun_scale(sol_scale.x0)) <= _ROOTS_TOL:
                 new_scale = sol_scale.x0
                 dl_list = np.array([small_dl * new_scale**i for i in range(num_step)])
                 dl_list = np.append(small_dl, dl_list)
