@@ -25,7 +25,7 @@ class Tidy3dConfig(pd.BaseModel):
         'Can be "DEBUG", "INFO", "WARNING", "ERROR", or "CRITICAL".',
     )
 
-    @pd.validator("logging_level", always=True)
+    @pd.validator("logging_level", pre=True, always=True)
     def _set_logging_level(cls, val):
         """Set the logging level if logging_level is changed."""
         set_logging_level(val)
