@@ -6,37 +6,39 @@ import csv
 import codecs
 import requests
 
-import scipy
+import scipy.optimize as opt
 import numpy as np
 from rich.progress import Progress
 from pydantic import Field, validator
 
+from ...log import log
 from ...components.base import Tidy3dBaseModel
-from ...components import PoleResidue, AbstractMedium
-from ...constants import C_0, HBAR, MICROMETER
+from ...components.medium import PoleResidue, AbstractMedium
 from ...components.viz import add_ax_if_none
-from ...components.types import Ax, ArrayLike
-from ...log import log, ValidationError, WebError, SetupError
+from ...components.types import Ax, ArrayFloat1D
+from ...constants import C_0, HBAR, MICROMETER
+from ...exceptions import ValidationError, WebError, SetupError
+from ...web.config import DEFAULT_CONFIG as Config
 
 
 class DispersionFitter(Tidy3dBaseModel):
     """Tool for fitting refractive index data to get a
     dispersive medium described by :class:`.PoleResidue` model."""
 
-    wvl_um: Union[Tuple[float, ...], ArrayLike[float, 1]] = Field(
+    wvl_um: Union[Tuple[float, ...], ArrayFloat1D] = Field(
         ...,
         title="Wavelength data",
         description="Wavelength data in micrometers.",
         units=MICROMETER,
     )
 
-    n_data: Union[Tuple[float, ...], ArrayLike[float, 1]] = Field(
+    n_data: Union[Tuple[float, ...], ArrayFloat1D] = Field(
         ...,
         title="Index of refraction data",
         description="Real part of the complex index of refraction.",
     )
 
-    k_data: Union[Tuple[float, ...], ArrayLike[float, 1]] = Field(
+    k_data: Union[Tuple[float, ...], ArrayFloat1D] = Field(
         None,
         title="Extinction coefficient data",
         description="Imaginary part of the complex index of refraction.",
@@ -478,10 +480,10 @@ class DispersionFitter(Tidy3dBaseModel):
         bounds = list(zip(bounds_lower, bounds_upper))
 
         # TODO: set up constraint properly
-        scipy_constraint = scipy.optimize.NonlinearConstraint(constraint, lb=0, ub=np.inf)
+        scipy_constraint = opt.NonlinearConstraint(constraint, lb=0, ub=np.inf)
 
         # TODO: set options properly
-        res = scipy.optimize.minimize(
+        res = opt.minimize(
             obj,
             coeffs0,
             args=(),
@@ -600,7 +602,7 @@ class DispersionFitter(Tidy3dBaseModel):
             raise ValidationError("Invalid URL. Too many k labels.")
 
     @classmethod
-    def from_url(cls, url_file: str, delimiter: str = ",", **kwargs):
+    def from_url(cls, url_file: str, delimiter: str = ",", ignore_k: bool = False, **kwargs):
         """loads :class:`DispersionFitter` from url linked to a csv/txt file that
         contains wavelength (micron), n, and optionally k data. Preferred from
         refractiveindex.info.
@@ -639,6 +641,8 @@ class DispersionFitter(Tidy3dBaseModel):
             e.g. "https://refractiveindex.info/data_csv.php?datafile=data/main/Ag/Johnson.yml"
         delimiter : str = ","
             E.g. in refractiveindex.info, it'll be "," for csv file, and "\\\\t" for txt file.
+        ignore_k : bool = False
+            Ignore the k data if they are present, so the fitted material is lossless.
 
         Returns
         -------
@@ -646,7 +650,7 @@ class DispersionFitter(Tidy3dBaseModel):
             A :class:`DispersionFitter` instance.
         """
 
-        resp = requests.get(url_file)
+        resp = requests.get(url_file, verify=Config.ssl_verify)
 
         try:
             resp.raise_for_status()
@@ -677,8 +681,8 @@ class DispersionFitter(Tidy3dBaseModel):
         n_lam = np.array(n_lam)
         k_lam = np.array(k_lam)
 
-        if has_k == 1:
-            if np.allclose(n_lam[:, 0], k_lam[:, 0]):
+        if has_k == 1 and not ignore_k:
+            if n_lam.shape == k_lam.shape and np.allclose(n_lam[:, 0], k_lam[:, 0]):
                 return cls(wvl_um=n_lam[:, 0], n_data=n_lam[:, 1], k_data=k_lam[:, 1], **kwargs)
             raise ValidationError(
                 "Invalid URL. Both n and k should be provided at each wavelength."
@@ -687,7 +691,7 @@ class DispersionFitter(Tidy3dBaseModel):
         return cls(wvl_um=n_lam[:, 0], n_data=n_lam[:, 1], **kwargs)
 
     @classmethod
-    def from_file(cls, fname: str, **loadtxt_kwargs):
+    def from_file(cls, fname: str, **loadtxt_kwargs):  # pylint:disable=arguments-differ
         """Loads :class:`DispersionFitter` from file containing wavelength, n, k data.
 
         Parameters
