@@ -12,7 +12,7 @@ from pydantic import Field, NonNegativeFloat, PositiveInt, validator
 
 from ...log import log
 from ...components.base import Tidy3dBaseModel
-from ...components.types import ArrayComplex1D, ArrayComplex2D, ArrayComplex3D
+from ...components.types import ArrayFloat1D, ArrayComplex1D, ArrayComplex2D, ArrayComplex3D
 from ...components.data.data_array import ScalarFieldTimeDataArray
 from ...components.data.monitor_data import FieldTimeData
 from ...constants import HERTZ
@@ -34,7 +34,7 @@ class ResonanceData(Tidy3dBaseModel):
     complex_amplitudes: ArrayComplex1D = Field(
         None, title="Complex amplitudes", description="Complex resonance amplitudes"
     )
-    errors: ArrayComplex1D = Field(
+    errors: ArrayFloat1D = Field(
         None, title="Errors", description="Rough eigenvalue error estimate."
     )
 
@@ -61,7 +61,7 @@ class ResonanceFinder(Tidy3dBaseModel):
     Example
     -------
     >>> import numpy as np
-    >>> from tidy3d.plugins import ResonanceFinder
+    >>> from tidy3d.plugins.resonance import ResonanceFinder
     >>> t = np.linspace(0, 10000, 10000)
     >>> f1 = 2*np.pi*0.1 - 1j*0.002
     >>> f2 = 2*np.pi*0.2 - 1j*0.0005
