@@ -1,7 +1,12 @@
 """ imports interfaces for interacting with server """
 import sys
 
+from .cli.migrate import migrate
 from .webapi import run, upload, get_info, start, monitor, delete, download, load, estimate_cost
-from .webapi import get_tasks, delete_old, download_json, download_log, load_simulation
+from .webapi import get_tasks, delete_old, download_json, download_log, load_simulation, real_cost
 from .container import Job, Batch, BatchData
 from .auth import get_credentials
+from .cli import tidy3d_cli
+from .asynchronous import run_async
+
+migrate()
