@@ -78,7 +78,7 @@ async def _run_async(
             task_name=task_name,
             callback_url=callback_url,
             folder_name=folder_name,
-            verbose=False,
+            verbose=False,  # note, set verbose=False here because we use a custom progressbar below
         )
         job.start()
 
