@@ -135,7 +135,8 @@ def upload(  # pylint:disable=too-many-locals,too-many-arguments
 
     task = SimulationTask.create(simulation, task_name, folder_name, callback_url)
     if verbose:
-        log.info(f"Created task '{task_name}' with task_id '{task.task_id}'.")
+        console = Console()
+        console.log(f"Created task '{task_name}' with task_id '{task.task_id}'.")
     task.upload_simulation(verbose=verbose, progress_callback=progress_callback)
 
     # log the url for the task in the web UI
@@ -234,6 +235,8 @@ def monitor(task_id: TaskId, verbose: bool = True) -> None:
 
     break_statuses = ("success", "error", "diverged", "deleted", "draft")
 
+    console = Console() if verbose else None
+
     def get_status() -> str:
         """Get status for this task."""
         task_info = get_info(task_id)
@@ -257,14 +260,13 @@ def monitor(task_id: TaskId, verbose: bool = True) -> None:
             if new_status != status:
                 status = new_status
                 if verbose and status != "running":
-                    log.info(f"status = {status}")
+                    console.log(f"status = {status}")
             time.sleep(REFRESH_TIME)
 
     status = get_status()
 
     if verbose:
-        console = Console()
-        log.info(f"status = {status}")
+        console.log(f"status = {status}")
 
     # already done
     if status in break_statuses:
@@ -281,11 +283,11 @@ def monitor(task_id: TaskId, verbose: bool = True) -> None:
     if verbose:
         est_flex_unit = get_estimated_cost()
         if est_flex_unit is not None and est_flex_unit > 0:
-            log.info(
-                f"Maximum FlexUnit cost: {est_flex_unit:1.3f}. Use 'web.real_cost(task_id)' to "
-                "get the billed FlexUnit cost after a simulation run."
+            console.log(
+                f"Maximum FlexCredit cost: {est_flex_unit:1.3f}. Use 'web.real_cost(task_id)' to "
+                "get the billed FlexCredit cost after a simulation run."
             )
-        log.info("starting up solver")
+        console.log("starting up solver")
 
     # while running but before the percentage done is available, keep waiting
     while get_run_info(task_id)[0] is None and get_status() == "running":
@@ -295,7 +297,7 @@ def monitor(task_id: TaskId, verbose: bool = True) -> None:
     if verbose:
 
         # verbose case, update progressbar
-        log.info("running solver")
+        console.log("running solver")
         with Progress(console=console) as progress:
 
             pbar_pd = progress.add_task("% done", total=100)
@@ -303,12 +305,12 @@ def monitor(task_id: TaskId, verbose: bool = True) -> None:
 
             while perc_done is not None and perc_done < 100 and get_status() == "running":
                 perc_done, field_decay = get_run_info(task_id)
-                new_description = f"% done (field decay = {field_decay:.2e})"
+                new_description = f"solver progress (field decay = {field_decay:.2e})"
                 progress.update(pbar_pd, completed=perc_done, description=new_description)
                 time.sleep(RUN_REFRESH_TIME)
 
             if perc_done is not None and perc_done < 100:
-                log.info("early shutoff detected, exiting.")
+                console.log("early shutoff detected, exiting.")
 
             progress.update(pbar_pd, completed=100, refresh=True)
 
@@ -325,14 +327,14 @@ def monitor(task_id: TaskId, verbose: bool = True) -> None:
 
         status = get_status()
         if status != "running":
-            log.info(f"status = {status}")
+            console.log(f"status = {status}")
 
         with console.status(f"[bold green]Finishing '{task_name}'...", spinner="runner"):
             while status not in break_statuses:
                 new_status = get_status()
                 if new_status != status:
                     status = new_status
-                    log.info(f"status = {status}")
+                    console.log(f"status = {status}")
                 time.sleep(REFRESH_TIME)
     else:
         while get_status() not in break_statuses:
@@ -467,9 +469,6 @@ def download_log(
     ----
     To load downloaded results into data, call :meth:`load` with option `replace_existing=False`.
     """
-    # task = SimulationTask.get(task_id)
-    # if not task:
-    #     raise ValueError("Task not found.")
     task = SimulationTask(taskId=task_id)
     task.get_log(path, verbose=verbose, progress_callback=progress_callback)
 
@@ -506,7 +505,8 @@ def load(
         download(task_id=task_id, path=path, verbose=verbose, progress_callback=progress_callback)
 
     if verbose:
-        log.info(f"loading SimulationData from {path}")
+        console = Console()
+        console.log(f"loading SimulationData from {path}")
 
     sim_data = SimulationData.from_file(path)
 
@@ -607,7 +607,7 @@ def get_tasks(
 
 
 def estimate_cost(task_id: str) -> float:
-    """Compute the maximum flex unit charge for a given task.
+    """Compute the maximum FlexCredit charge for a given task.
 
     Parameters
     ----------
@@ -632,7 +632,7 @@ def estimate_cost(task_id: str) -> float:
     Returns
     -------
     float
-        Estimated cost of the task in flex units.
+        Estimated cost of the task in FlexCredits.
     """
 
     task = SimulationTask.get(task_id)
@@ -660,7 +660,7 @@ def real_cost(task_id: str) -> float:
     flex_unit = task_info.realFlexUnit
     if not flex_unit:
         log.warning(
-            f"Billed FlexUnit for task '{task_id}' is not available. If the task has been "
+            f"Billed FlexCredit for task '{task_id}' is not available. If the task has been "
             "successfully run, it should be available shortly."
         )
     return flex_unit
